@@ -1,3 +1,8 @@
+.splitMatrix <- function(dataMatrix, grps){
+  idx <- 1:nrow(dataMatrix)
+  ggg <- split(idx, grps)
+  lapply(ggg, function(iii) dataMatrix[iii,,drop=FALSE])
+}
 
 splitBy <-function (formula, data = parent.frame(),drop=TRUE, return.matrix=FALSE)
 {
@@ -65,11 +70,17 @@ splitBy <-function (formula, data = parent.frame(),drop=TRUE, return.matrix=FALS
             
             grps <- grpsvec ## 
 
+#            print(grps)
             
             dataMatrix <- .asNumericMatrix2(data)
-            at    <- .subsAttr2(data)
-            alist <- mApply(dataMatrix, grps, function(x){x}, simplify=FALSE)
 
+#            grps <<- grps
+#            dataMatrix <<- dataMatrix
+            
+            at    <- .subsAttr2(data)
+            #alist <- mApply(dataMatrix, grps, function(x){x}, simplify=FALSE)
+            alist <- .splitMatrix(dataMatrix,grps)
+            
             if (drop==TRUE)
               alist <- alist[lapply(alist,nrow)>0]
             
@@ -110,14 +121,6 @@ splitBy <-function (formula, data = parent.frame(),drop=TRUE, return.matrix=FALS
   }
 
 
-
- #       m     <- match.call(expand.dots = TRUE)
-
-#    m <- match(c("formula", "data", "subset", "weights", "na.action",
-#                 "offset"), names(mf), 0)
-
-
-
 print.splitByData <- function(x,...){
 #  print(attr(x,"groupid"))
   print(cbind(listentry=names(x), attr(x,"groupid")))
@@ -127,88 +130,88 @@ print.splitByData <- function(x,...){
 
 
 
-.splitByOLD<-function (formula, data = parent.frame(),drop=TRUE, return.matrix=FALSE)
-{
-    mf <- match.call(expand.dots = FALSE)
+## .splitByOLD<-function (formula, data = parent.frame(),drop=TRUE, return.matrix=FALSE)
+## {
+##     mf <- match.call(expand.dots = FALSE)
     
-    m <- match(c("formula", "data", "subset", "weights", "na.action",
-                 "offset"), names(mf), 0)
+##     m <- match(c("formula", "data", "subset", "weights", "na.action",
+##                  "offset"), names(mf), 0)
 
-    ff <- as.formula(eval.parent(mf[[2]]))
+##     ff <- as.formula(eval.parent(mf[[2]]))
 
-    if (ff[[2]]==1){
-      groupData <- list(data)
-      names(groupData) <- "1"
-      groupid <- as.data.frame(1)
-      idxvec  <- list(1:nrow(data))
-      grps    <- "1"
+##     if (ff[[2]]==1){
+##       groupData <- list(data)
+##       names(groupData) <- "1"
+##       groupid <- as.data.frame(1)
+##       idxvec  <- list(1:nrow(data))
+##       grps    <- "1"
 
-      ## attr(groupData,"groupid") <- 1
-    } else { 
-      ff    <- terms(ff, data = data)
-      m     <- match.call(expand.dots = TRUE)
-      group <- attr(terms(ff), "term.labels")
+##       ## attr(groupData,"groupid") <- 1
+##     } else { 
+##       ff    <- terms(ff, data = data)
+##       m     <- match.call(expand.dots = TRUE)
+##       group <- attr(terms(ff), "term.labels")
       
-      ## workinggroup: Those variables in 'group' which are **not** constant.
-      ##
-      nonconstgroup <- rep(TRUE, length(group))      
-      names(nonconstgroup) <- group
-      for (i in 1:length(group)){
-        nunique <- length(unique(data[,group[i]]))
-        if (nunique==1)
-          nonconstgroup[i] <- FALSE
-      }
-      workinggroup <- group[nonconstgroup]            
+##       ## workinggroup: Those variables in 'group' which are **not** constant.
+##       ##
+##       nonconstgroup <- rep(TRUE, length(group))      
+##       names(nonconstgroup) <- group
+##       for (i in 1:length(group)){
+##         nunique <- length(unique(data[,group[i]]))
+##         if (nunique==1)
+##           nonconstgroup[i] <- FALSE
+##       }
+##       workinggroup <- group[nonconstgroup]            
       
-      if (length(workinggroup)==0){
-        groupData <- list(data)
-        names(groupData) <- "1"
-        groupid <- data[1,group,drop=FALSE]
-        idxvec  <- list(1:nrow(data))
-        grps    <- "1"
-                                        #attr(groupData,"groupid") <- data[1,group]
-      } else {
-        ## grps: Recode groups into one vector; aa|b|xx etc...
-        ## 
-        grps <- data[, workinggroup,drop=FALSE]
-        grpsvec<-paste(grps[,1])
-        if (ncol(grps)>1){
-          for (j in 2:ncol(grps)){
-            grpsvec <- paste(paste(grps[,j]),grpsvec,sep='|')
-          }
-        }
-        grps <- grpsvec ## 
-        dataMatrix <- .asNumericMatrix2(data)
-        at <- .subsAttr2(data)        
-        a <- mApply(dataMatrix, grps, function(x){x}, simplify=FALSE)
-        if (drop==TRUE)
-          a<- a[lapply(a,nrow)>0]
+##       if (length(workinggroup)==0){
+##         groupData <- list(data)
+##         names(groupData) <- "1"
+##         groupid <- data[1,group,drop=FALSE]
+##         idxvec  <- list(1:nrow(data))
+##         grps    <- "1"
+##                                         #attr(groupData,"groupid") <- data[1,group]
+##       } else {
+##         ## grps: Recode groups into one vector; aa|b|xx etc...
+##         ## 
+##         grps <- data[, workinggroup,drop=FALSE]
+##         grpsvec<-paste(grps[,1])
+##         if (ncol(grps)>1){
+##           for (j in 2:ncol(grps)){
+##             grpsvec <- paste(paste(grps[,j]),grpsvec,sep='|')
+##           }
+##         }
+##         grps <- grpsvec ## 
+##         dataMatrix <- .asNumericMatrix2(data)
+##         at <- .subsAttr2(data)        
+##         a <- mApply(dataMatrix, grps, function(x){x}, simplify=FALSE)
+##         if (drop==TRUE)
+##           a<- a[lapply(a,nrow)>0]
         
-        if (return.matrix==TRUE)
-          groupData <- a
-        else{
-          groupData <- lapply(a, .matrix2dataFrame2, at=at,restoreAll=FALSE)
-        }
-        groupid        <- lapply(groupData, function(x) x[1,group,drop=FALSE])
+##         if (return.matrix==TRUE)
+##           groupData <- a
+##         else{
+##           groupData <- lapply(a, .matrix2dataFrame2, at=at,restoreAll=FALSE)
+##         }
+##         groupid        <- lapply(groupData, function(x) x[1,group,drop=FALSE])
 
-        names(groupid) <- NULL
-        groupid        <- as.data.frame(do.call('rbind',groupid))
-        names(groupid) <- group
-        rownames(groupid) <- 1:nrow(groupid)     
-        uniq.grps <- unique(grps)
-        idxvec <- as.list(rep(NA, length(uniq.grps)))
-        names(idxvec) <- uniq.grps
-        for (ii in 1:length(uniq.grps)){
-          idxvec[[ii]] <- which(grps==uniq.grps[ii])
-         }      
+##         names(groupid) <- NULL
+##         groupid        <- as.data.frame(do.call('rbind',groupid))
+##         names(groupid) <- group
+##         rownames(groupid) <- 1:nrow(groupid)     
+##         uniq.grps <- unique(grps)
+##         idxvec <- as.list(rep(NA, length(uniq.grps)))
+##         names(idxvec) <- uniq.grps
+##         for (ii in 1:length(uniq.grps)){
+##           idxvec[[ii]] <- which(grps==uniq.grps[ii])
+##          }      
         
-      }
-    }
+##       }
+##     }
     
-    attr(groupData,"groupid") <- groupid
-    attr(groupData,"idxvec")  <- idxvec
-    attr(groupData,"grps")    <- grps
+##     attr(groupData,"groupid") <- groupid
+##     attr(groupData,"idxvec")  <- idxvec
+##     attr(groupData,"grps")    <- grps
     
-    class(groupData) <- c("splitByData", "list")
-    return(groupData)
-}
+##     class(groupData) <- c("splitByData", "list")
+##     return(groupData)
+## }
